/*********************************************************************
** File:			sqltypes.h
**
** Copyright:		Copyright (c) 1995-1999 Microsoft Corporation
**
** Contents: 		This file defines the types used in ODBC
**
** Comments: 		
**
*********************************************************************/

#ifndef __SQLTYPES
#define __SQLTYPES

/*
 *  __freebsd__ is used internally at Easysoft but
 *  cc on FreeBSD defines __FreeBSD__
 */
#if !defined(EMBEDDED) && !defined(__freebsd__) && !defined(__FreeBSD__) && !defined(__darwin__)
#include <wchar.h>
#endif

/* if ODBCVER is not defined, assume version 3.51 */
#ifndef ODBCVER
#define ODBCVER	0x0351
#endif  /* ODBCVER */

#ifdef __cplusplus
extern "C" { 			/* Assume C declarations for C++   */
#endif  /* __cplusplus */

/* environment specific definitions */
#ifndef EXPORT
#define EXPORT
#endif

#ifdef WIN32
# define SQL_API  __stdcall
#else
# define SQL_API
# if defined(__sgi) && (_SIZE_LONG == 64)
#  define __IS64BIT 1
# endif
/* Various types used throughout that Windows defines and UNIX doesn't */
# if defined(__alpha) || defined(__sparcv9) || defined(__LP64__) || defined(__IS64BIT)
    typedef int DWORD;
# else
    typedef long DWORD;
# endif
    typedef short WORD;
    typedef char  BYTE;
    typedef const char* LPCSTR;
    typedef char* LPSTR;
    typedef DWORD* LPDWORD;
#endif

#ifndef RC_INVOKED

/* API declaration data types */
typedef unsigned char   SQLCHAR;
#if (ODBCVER >= 0x0300)
typedef signed char     SQLSCHAR;
typedef unsigned char   SQLDATE;
typedef unsigned char   SQLDECIMAL;
typedef double          SQLDOUBLE;
typedef double          SQLFLOAT;
#endif
#if defined(__alpha) || defined(__sparcv9) || defined(__LP64__) || defined(__IS64BIT)
typedef int            SQLINTEGER;
typedef unsigned int   SQLUINTEGER;
#else
/* In Win64 long is 32bits! so MS appear to be fixing SQLINTEGER ar 32 bits*/    
typedef long            SQLINTEGER;
typedef unsigned long   SQLUINTEGER;
#endif

#define SQLROWSETSIZE   SQLUINTEGER

#ifdef _WIN64
typedef INT64           SQLLEN;
typedef INT64           SQLROWOFFSET;
typedef UINT64          SQLROWCOUNT;
typedef UINT64          SQLULEN;
typedef UINT64          SQLTRANSID;
typedef unsigned long   SQLSETPOSIROW;
#else
/*
 *  The way MDACK SDK 2.6 is going the following commented out defns are
 *  correct but this has such a huge impact and MS's help/odbc documentation
 *  is still contradictory so we fix each at 32bits for now. Waiting on
 *  clarification.
 *
 *  #define SQLLEN          long
 *  #define SQLULEN         unsigned long
 *  #define SQLROWOFFSET    long
 *  #define SQLROWCOUNT     unsigned long
 */
#if defined(__alpha) || defined(__sparcv9) || defined(__LP64__) || defined(__IS64BIT)
# define SQLLEN int
# define SQLULEN unsigned int
# define SQLROWOFFSET int
# define SQLROWCOUNT unsigned int
#else
# define SQLLEN long
# define SQLULEN unsigned long
# define SQLROWOFFSET long
# define SQLROWCOUNT unsigned long
#endif
    
#define SQLTRANSID      DWORD
#define SQLSETPOSIROW   SQLUSMALLINT
#endif

#if (ODBCVER >= 0x0300)
typedef unsigned char   SQLNUMERIC;
#endif
typedef void *          SQLPOINTER;
#if (ODBCVER >= 0x0300)
typedef float           SQLREAL;
#endif
typedef short           SQLSMALLINT;
typedef unsigned short  SQLUSMALLINT;
#if (ODBCVER >= 0x0300)
typedef unsigned char   SQLTIME;
typedef unsigned char   SQLTIMESTAMP;
typedef unsigned char   SQLVARCHAR;
#endif

/* function return type */
typedef SQLSMALLINT     SQLRETURN;

/* generic data structures */
#if (ODBCVER >= 0x0300)
/*
 *  mje, commented out following to ensure SQLHANDLE was void*
 *  It appears MS's 64bit compilers define WIN32 so the second defn
 *  would only get used on non-Windows platforms.
 */
   
/*#if defined(WIN32)*/
typedef void*					SQLHANDLE;
/*#else*/
/*typedef SQLINTEGER		SQLHANDLE;*/
/*#endif*/	/* defined(WIN32) */
typedef SQLHANDLE               SQLHENV;
typedef SQLHANDLE               SQLHDBC;
typedef SQLHANDLE               SQLHSTMT;
typedef SQLHANDLE               SQLHDESC;
#else
#if defined(WIN32)
typedef void*					SQLHENV;
typedef void*					SQLHDBC;
typedef void*					SQLHSTMT;
#else
typedef SQLINTEGER              SQLHENV;
typedef SQLINTEGER              SQLHDBC;
typedef SQLINTEGER              SQLHSTMT;
#endif  /* defined(WIN32) */
#endif /* ODBCVER >= 0x0300 */

/* SQL portable types for C */
typedef unsigned char           UCHAR;
typedef signed char             SCHAR;
typedef short int               SWORD;
typedef unsigned short int      UWORD;
#if defined(__alpha) || defined(__sparcv9) || defined(__LP64__) || defined(__IS64BIT)
typedef unsigned int       UDWORD;
typedef int                SDWORD;
#else
typedef unsigned long int       UDWORD;
typedef long int                SDWORD;
#endif
#ifdef WIN32
typedef SCHAR                   SQLSCHAR;
typedef UDWORD                  SQLUINTEGER;
#endif
#if defined(__alpha) || defined(__sparcv9) || defined(__LP64__) || defined(__IS64BIT)
typedef signed int             SLONG;
typedef unsigned int           ULONG;
#else
typedef signed long             SLONG;
typedef unsigned long           ULONG;
#endif
typedef signed short            SSHORT;
typedef unsigned short          USHORT;
typedef double                  SDOUBLE;
typedef double            		LDOUBLE; 
typedef float                   SFLOAT;

typedef void*              		PTR;

#if (ODBCVER >= 0x0300)
typedef void *                        HENV;
typedef void *                        HDBC;
typedef void *                        HSTMT;
#else
typedef SQLINTEGER                        HENV;
typedef SQLINTEGER                        HDBC;
typedef SQLINTEGER                        HSTMT;
#endif

typedef signed short            RETCODE;

#if defined(WIN32) || defined(OS2)
typedef HWND                    SQLHWND;
#elif defined (UNIX)
/* SQLHWND was Widget in MS headers */
typedef void*                           HWND;
typedef void *                  SQLHWND;
#else
/* placehold for future O/S GUI window handle definition */
typedef void*                           HWND;
typedef SQLPOINTER              SQLHWND;
#endif

#ifndef	__SQLDATE
#define	__SQLDATE
/* transfer types for DATE, TIME, TIMESTAMP */
typedef struct tagDATE_STRUCT
{
        SQLSMALLINT    year;
        SQLUSMALLINT   month;
        SQLUSMALLINT   day;
} DATE_STRUCT;

#if (ODBCVER >= 0x0300)
typedef DATE_STRUCT	SQL_DATE_STRUCT;
#endif  /* ODBCVER >= 0x0300 */

typedef struct tagTIME_STRUCT
{
        SQLUSMALLINT   hour;
        SQLUSMALLINT   minute;
        SQLUSMALLINT   second;
} TIME_STRUCT;

#if (ODBCVER >= 0x0300)
typedef TIME_STRUCT	SQL_TIME_STRUCT;
#endif /* ODBCVER >= 0x0300 */

typedef struct tagTIMESTAMP_STRUCT
{
        SQLSMALLINT    year;
        SQLUSMALLINT   month;
        SQLUSMALLINT   day;
        SQLUSMALLINT   hour;
        SQLUSMALLINT   minute;
        SQLUSMALLINT   second;
        SQLUINTEGER    fraction;
} TIMESTAMP_STRUCT;

#if (ODBCVER >= 0x0300)
typedef TIMESTAMP_STRUCT	SQL_TIMESTAMP_STRUCT;
#endif  /* ODBCVER >= 0x0300 */


/*
 * enumerations for DATETIME_INTERVAL_SUBCODE values for interval data types
 * these values are from SQL-92
 */

#if (ODBCVER >= 0x0300)
typedef enum 
{
	SQL_IS_YEAR						= 1,
	SQL_IS_MONTH					= 2,
	SQL_IS_DAY						= 3,
	SQL_IS_HOUR						= 4,
	SQL_IS_MINUTE					= 5,
	SQL_IS_SECOND					= 6,
	SQL_IS_YEAR_TO_MONTH			= 7,
	SQL_IS_DAY_TO_HOUR				= 8,
	SQL_IS_DAY_TO_MINUTE			= 9,
	SQL_IS_DAY_TO_SECOND			= 10,
	SQL_IS_HOUR_TO_MINUTE			= 11,
	SQL_IS_HOUR_TO_SECOND			= 12,
	SQL_IS_MINUTE_TO_SECOND			= 13
} SQLINTERVAL;

#endif  /* ODBCVER >= 0x0300 */

#if (ODBCVER >= 0x0300)
typedef struct tagSQL_YEAR_MONTH
{
		SQLUINTEGER		year;
		SQLUINTEGER		month;
} SQL_YEAR_MONTH_STRUCT;

typedef struct tagSQL_DAY_SECOND
{
		SQLUINTEGER		day;
		SQLUINTEGER		hour;
		SQLUINTEGER		minute;
		SQLUINTEGER		second;
		SQLUINTEGER		fraction;
} SQL_DAY_SECOND_STRUCT;

typedef struct tagSQL_INTERVAL_STRUCT
{
	SQLINTERVAL		interval_type;
	SQLSMALLINT		interval_sign;
	union {
		SQL_YEAR_MONTH_STRUCT		year_month;
		SQL_DAY_SECOND_STRUCT		day_second;
	} intval;

} SQL_INTERVAL_STRUCT;

#endif  /* ODBCVER >= 0x0300 */

#endif	/* __SQLDATE	*/

/* the ODBC C types for SQL_C_SBIGINT and SQL_C_UBIGINT */
#if (ODBCVER >= 0x0300)
# if (_MSC_VER >= 900)
#   define ODBCINT64	__int64
#   define UODBCINT64	unsigned __int64
# else
#   if defined(__alpha) || defined(__sparcv9) || defined(__LP64__) || defined(__IS64BIT)
#     define ODBCINT64 long
#     define UODBCINT64 unsigned long
#   elif defined(_SCO_ELF) || defined (__USLC__)
typedef struct __bigint_struct
{
    int             hiword;
    unsigned int    loword;
} __bigint_struct_t;
typedef struct __ubigint_struct
{
    unsigned int    hiword;
    unsigned int    loword;
} __ubigint_struct_t;
#  define ODBCINT64     __bigint_struct_t
#  define UODBCINT64    __ubigint_struct_t

#   else
#     define ODBCINT64 long long
#     define UODBCINT64 unsigned long long
#   endif
#  endif
/* If using other compilers, define ODBCINT64 to the 
	approriate 64 bit integer type */
#ifdef ODBCINT64
typedef ODBCINT64	SQLBIGINT;
#endif
#ifdef UODBCINT64
typedef UODBCINT64	SQLUBIGINT;
#endif
#endif  /* ODBCVER >= 0x0300 */

/* internal representation of numeric data type */
#if (ODBCVER >= 0x0300)
#define SQL_MAX_NUMERIC_LEN		16
typedef struct tagSQL_NUMERIC_STRUCT
{
	SQLCHAR		precision;
	SQLSCHAR	scale;
	SQLCHAR		sign;	/* 1 if positive, 0 if negative */
	SQLCHAR		val[SQL_MAX_NUMERIC_LEN];
} SQL_NUMERIC_STRUCT;
#endif  /* ODBCVER >= 0x0300 */

#if (ODBCVER >= 0x0350)
 #ifdef GUID_DEFINED
  typedef GUID	SQLGUID;
 #else
  #ifndef WIN32
   /* size is 16 */
   #ifndef WITHOUT_GUID
    typedef struct  tagSQLGUID
    {
        DWORD Data1;
        WORD Data2;
        WORD Data3;
        BYTE Data4[ 8 ];
    } SQLGUID;
   #endif /* WITHOUT_GUID */
  #endif /* WIN32 */
 #endif  /* GUID_DEFINED */
#endif  /* ODBCVER >= 0x0350 */

#if defined(__alpha) || defined(__sparcv9) || defined(__LP64__) || defined(__IS64BIT)
/*  Bookmark is a 32bit quantity */
typedef unsigned int       BOOKMARK;
#else
typedef unsigned long int       BOOKMARK;
#endif

/*
 *  The logical thing here would be to define SQLWCHAR as wchar_t if
 *  _WCHAR_T_DEFINED is defined. However, wchar_t tends to be a different
 *  size on every platform and most ODBC driver vendors have standardised
 *  on a 2 byte quantity for SQLWCHAR (e.g. to hold UTF16). If you want the
 *  old behavior then define SQL_WCHART_CONVERT but we would not recommend it.
 */
#if (defined(SQL_WCHART_CONVERT) && defined(_WCHAR_T_DEFINED))
typedef wchar_t SQLWCHAR;
#else
typedef unsigned short SQLWCHAR;
#endif

#ifdef UNICODE
typedef SQLWCHAR        SQLTCHAR;
#else
typedef SQLCHAR         SQLTCHAR;
#endif  /* UNICODE */



#endif     /* RC_INVOKED */


#ifdef __cplusplus
}                                    /* End of extern "C" { */
#endif  /* __cplusplus */

#endif /* #ifndef __SQLTYPES */
