/*
 *  $Id: SQLDataSources.c 91 2004-05-20 15:05:37Z martin $
 *
 *  $Log: SQLDataSources.c,v $
 *  Revision 1.2  2003/09/04 07:54:38  martin
 *  Make this work on Windows.
 *
 *  Revision 1.1  2003/06/19 13:20:08  martin
 *  Initial revision
 *
 *
 */
#ifdef WIN32
# include <windows.h>
#endif

#include <stdio.h>

#include <sql.h>
#include <sqlext.h>

static void extract_error(
    char *fn,
    SQLHANDLE handle,
    SQLSMALLINT type);

main()
{
    SQLHENV             henv;
    SQLRETURN           ret;
    SQLUSMALLINT        dir;
    SQLCHAR             dsn_name[100];
    SQLSMALLINT         name_len;
    SQLCHAR             dsn_desc[100];
    SQLSMALLINT         desc_len;

    dir = SQL_FETCH_FIRST;
    
    ret = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
    if (ret != SQL_SUCCESS) {
        extract_error("SQLAllocHandle(SQL_HANDLE_ENV)", henv, SQL_HANDLE_ENV);
        return 1;
    }
    
    ret = SQLSetEnvAttr(henv, SQL_ATTR_ODBC_VERSION, (void *)SQL_OV_ODBC3, 0);
    if (ret != SQL_SUCCESS) {
        extract_error("SQLSetEnvAttr(SQL_OV_ODBC3)", henv, SQL_HANDLE_ENV);
        SQLFreeHandle(SQL_HANDLE_ENV, henv);
        return 2;
    }
    
    while(SQL_SUCCEEDED(ret =
              SQLDataSources(henv, dir, dsn_name, sizeof(dsn_name), &name_len,
                             dsn_desc, sizeof(dsn_desc), &desc_len)))
    {
        dir = SQL_FETCH_NEXT;

        printf("%s, %s\n", dsn_name, dsn_desc);
    }
    if ((!SQL_SUCCEEDED(ret)) && (ret != SQL_NO_DATA)) {
        extract_error("SQLDataSources", henv, SQL_HANDLE_ENV);
        SQLFreeHandle(SQL_HANDLE_ENV, henv);
        return 3;
    }
    
    SQLFreeHandle(SQL_HANDLE_ENV, henv);

    return 0;
    
}


static void extract_error(
    char *fn,
    SQLHANDLE handle,
    SQLSMALLINT type)
{
    SQLINTEGER i = 0, native;
    SQLCHAR state[ 7 ];
    SQLCHAR text[256];
    SQLSMALLINT len;
    int ret;

    fprintf(stderr,
            "\n"
            "The driver reported the following diagnostics whilst running "
            "%s\n\n",
            fn);
    
    do
    {
        ret = SQLGetDiagRec(type, handle, ++i, state, &native, text,
                            sizeof(text), &len );
        if (SQL_SUCCEEDED(ret))
            printf( "%s:%ld:%ld:%s\n", state, i, native, text );
    }
    while( ret == SQL_SUCCESS );
}
